@@ -259,9 +259,13 @@ out:
  * cd_util_set_info_text:
  **/
 static gboolean
-cd_util_set_info_text (CdUtilPrivate *priv, cmsTagSignature sig, gchar **values, GError **error)
+cd_util_set_info_text (CdUtilPrivate *priv,
+		       cmsTagSignature sig,
+		       gchar **values,
+		       GError **error)
 {
 	cmsHPROFILE lcms_profile = NULL;
+	const gchar *value;
 	gboolean ret = TRUE;
 
 	/* check arguments */
@@ -279,10 +283,18 @@ cd_util_set_info_text (CdUtilPrivate *priv, cmsTagSignature sig, gchar **values,
 		goto out;
 	}
 
+	/* these are default values */
+	if (sig == cmsSigCopyrightTag &&
+	    g_strcmp0 (values[0], "") == 0) {
+		value = CD_PROFILE_DEFAULT_COPYRIGHT_STRING;
+	} else {
+		value = values[0];
+	}
+
 	/* update value */
 	ret = cd_util_profile_set_text_acsii (lcms_profile,
 					      sig,
-					      values[0],
+					      value,
 					      error);
 	if (!ret)
 		goto out;
@@ -303,7 +315,7 @@ out:
 static gboolean
 cd_util_set_copyright (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	return cd_util_set_info_text (priv, cmsInfoCopyright, values, error);
+	return cd_util_set_info_text (priv, cmsSigCopyrightTag, values, error);
 }
 
 /**
@@ -384,11 +396,14 @@ cd_util_get_standard_space_filename (CdUtilPrivate *priv,
 				     CdStandardSpace standard_space,
 				     GError **error)
 {
-	CdProfile *profile_tmp;
+	CdProfile *profile_tmp = NULL;
 	gboolean ret;
 	gchar *filename = NULL;
 
 	/* try to find */
+	ret = cd_client_connect_sync (priv->client, NULL, error);
+	if (!ret)
+		goto out;
 	profile_tmp = cd_client_get_standard_space_sync (priv->client,
 							 standard_space,
 							 NULL,
@@ -549,6 +564,51 @@ out:
 	return coverage;
 }
 
+/**
+ * cd_util_set_version:
+ **/
+static gboolean
+cd_util_set_version (CdUtilPrivate *priv, gchar **values, GError **error)
+{
+	cmsHPROFILE lcms_profile = NULL;
+	gboolean ret = TRUE;
+	gdouble version;
+
+	/* check arguments */
+	if (g_strv_length (values) != 2) {
+		ret = FALSE;
+		g_set_error_literal (error, 1, 0,
+				     "invalid input, expect 'filename' 'version'");
+		goto out;
+	}
+
+	/* get version */
+	version = atof (values[1]);
+	if (version < 1.0 || version > 6.0) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "invalid version %f", version);
+		goto out;
+	}
+
+	/* open profile and set version */
+	lcms_profile = cd_util_profile_read (values[0], error);
+	if (lcms_profile == NULL) {
+		ret = FALSE;
+		goto out;
+	}
+	cmsSetProfileVersion (lcms_profile, version);
+
+	/* write new file */
+	ret = cd_util_profile_write (lcms_profile, values[0], error);
+	if (!ret)
+		goto out;
+out:
+	if (lcms_profile != NULL)
+		cmsCloseProfile (lcms_profile);
+	return ret;
+}
+
 /**
  * cd_util_set_fix_metadata:
  **/
@@ -896,8 +956,10 @@ out:
 static gboolean
 cd_util_dump (CdUtilPrivate *priv, gchar **values, GError **error)
 {
+	cmsCIExyY yxy;
 	cmsHANDLE dict;
 	cmsHPROFILE lcms_profile = NULL;
+	const cmsCIEXYZ *xyz;
 	const cmsDICTentry* entry;
 	gboolean ret = TRUE;
 	gchar ascii_name[1024];
@@ -951,6 +1013,28 @@ cd_util_dump (CdUtilPrivate *priv, gchar **values, GError **error)
 		}
 	}
 
+	/* show Yxy primaries */
+	xyz = cmsReadTag (lcms_profile, cmsSigRedColorantTag);
+	if (xyz != NULL) {
+		cmsXYZ2xyY (&yxy, xyz);
+		g_print ("%s:\t%0.3f, %0.3f\n", _("Red primary"), yxy.x, yxy.y);
+	}
+	xyz = cmsReadTag (lcms_profile, cmsSigGreenColorantTag);
+	if (xyz != NULL) {
+		cmsXYZ2xyY (&yxy, xyz);
+		g_print ("%s:\t%0.3f, %0.3f\n", _("Green primary"), yxy.x, yxy.y);
+	}
+	xyz = cmsReadTag (lcms_profile, cmsSigBlueColorantTag);
+	if (xyz != NULL) {
+		cmsXYZ2xyY (&yxy, xyz);
+		g_print ("%s:\t%0.3f, %0.3f\n", _("Blue primary"), yxy.x, yxy.y);
+	}
+	xyz = cmsReadTag (lcms_profile, cmsSigMediaWhitePointTag);
+	if (xyz != NULL) {
+		cmsXYZ2xyY (&yxy, xyz);
+		g_print ("%s:\t%0.3f, %0.3f\n", _("Whitepoint"), yxy.x, yxy.y);
+	}
+
 	/* success */
 	ret = TRUE;
 out:
@@ -1077,12 +1161,6 @@ main (int argc, char *argv[])
 	/* create helper object */
 	priv = g_new0 (CdUtilPrivate, 1);
 	priv->client = cd_client_new ();
-	ret = cd_client_connect_sync (priv->client, NULL, &error);
-	if (!ret) {
-		g_print ("%s\n", error->message);
-		g_error_free (error);
-		goto out;
-	}
 
 	/* add commands */
 	priv->cmd_array = g_ptr_array_new_with_free_func ((GDestroyNotify) cd_util_item_free);
@@ -1141,6 +1219,11 @@ main (int argc, char *argv[])
 		     /* TRANSLATORS: command description */
 		     _("Automatically fix metadata in the profile"),
 		     cd_util_set_fix_metadata);
+	cd_util_add (priv->cmd_array,
+		     "set-version",
+		     /* TRANSLATORS: command description */
+		     _("Set the ICC profile version"),
+		     cd_util_set_version);
 
 	/* sort by command name */
 	g_ptr_array_sort (priv->cmd_array,
@@ -1154,7 +1237,15 @@ main (int argc, char *argv[])
 	/* TRANSLATORS: program name */
 	g_set_application_name (_("Color Management"));
 	g_option_context_add_main_entries (priv->context, options, NULL);
-	g_option_context_parse (priv->context, &argc, &argv, NULL);
+	ret = g_option_context_parse (priv->context, &argc, &argv, &error);
+	if (!ret) {
+		/* TRANSLATORS: the user didn't read the man page */
+		g_print ("%s: %s\n",
+			 _("Failed to parse arguments"),
+			 error->message);
+		g_error_free (error);
+		goto out;
+	}
 
 	/* set verbose? */
 	if (verbose) {
