@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2011 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2012 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -67,18 +67,6 @@ typedef struct
 	void (*_cd_device_reserved8) (void);
 } CdDeviceClass;
 
-/**
- * CdDeviceError:
- * @CD_DEVICE_ERROR_FAILED: the transaction failed for an unknown reason
- *
- * Errors that can be thrown
- */
-typedef enum
-{
-	CD_DEVICE_ERROR_FAILED,
-	CD_DEVICE_ERROR_LAST
-} CdDeviceError;
-
 GType		 cd_device_get_type			(void);
 GQuark		 cd_device_error_quark			(void);
 CdDevice	*cd_device_new				(void);
@@ -133,6 +121,14 @@ void		 cd_device_profiling_inhibit		(CdDevice	*device,
 gboolean	 cd_device_profiling_inhibit_finish	(CdDevice	*device,
 							 GAsyncResult	*res,
 							 GError		**error);
+void		 cd_device_set_enabled			(CdDevice	*device,
+							 gboolean	 enabled,
+							 GCancellable	*cancellable,
+							 GAsyncReadyCallback callback,
+							 gpointer	 user_data);
+gboolean	 cd_device_set_enabled_finish		(CdDevice	*device,
+							 GAsyncResult	*res,
+							 GError		**error);
 void		 cd_device_profiling_uninhibit		(CdDevice	*device,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
@@ -162,12 +158,15 @@ const gchar	*cd_device_get_id			(CdDevice	*device);
 const gchar	*cd_device_get_model			(CdDevice	*device);
 const gchar	*cd_device_get_vendor			(CdDevice	*device);
 const gchar	*cd_device_get_serial			(CdDevice	*device);
+const gchar	*cd_device_get_seat			(CdDevice	*device);
 const gchar	*cd_device_get_format			(CdDevice	*device);
+const gchar	**cd_device_get_profiling_inhibitors	(CdDevice	*device);
 guint64		 cd_device_get_created			(CdDevice	*device);
 guint64		 cd_device_get_modified			(CdDevice	*device);
 CdDeviceKind	 cd_device_get_kind			(CdDevice	*device);
 CdColorspace	 cd_device_get_colorspace		(CdDevice	*device);
 CdDeviceMode	 cd_device_get_mode			(CdDevice	*device);
+gboolean	 cd_device_get_enabled			(CdDevice	*device);
 CdObjectScope	 cd_device_get_scope			(CdDevice	*device);
 guint		 cd_device_get_owner			(CdDevice	*device);
 GPtrArray	*cd_device_get_profiles			(CdDevice	*device);
