@@ -24,6 +24,7 @@
 #include <gio/gio.h>
 #include <glib-object.h>
 #include <sqlite3.h>
+#include <syslog.h>
 
 #include "cd-common.h"
 #include "cd-device-db.h"
@@ -65,12 +66,13 @@ cd_device_db_load (CdDeviceDb *ddb,
 		goto out;
 
 	g_debug ("CdDeviceDb: trying to open database '%s'", filename);
+	syslog (LOG_INFO, "Using device database file %s", filename);
 	rc = sqlite3_open (filename, &ddb->priv->db);
 	if (rc != SQLITE_OK) {
 		ret = FALSE;
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "Can't open database: %s\n",
 			     sqlite3_errmsg (ddb->priv->db));
 		sqlite3_close (ddb->priv->db);
@@ -123,8 +125,8 @@ cd_device_db_empty (CdDeviceDb *ddb,
 	if (rc != SQLITE_OK) {
 		ret = FALSE;
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "SQL error: %s",
 			     error_msg);
 		sqlite3_free (error_msg);
@@ -159,8 +161,8 @@ cd_device_db_add (CdDeviceDb *ddb,
 	rc = sqlite3_exec (ddb->priv->db, statement, NULL, NULL, &error_msg);
 	if (rc != SQLITE_OK) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "SQL error: %s",
 			     error_msg);
 		sqlite3_free (error_msg);
@@ -191,7 +193,7 @@ cd_device_db_set_property (CdDeviceDb *ddb,
 	g_return_val_if_fail (ddb->priv->db != NULL, FALSE);
 
 	g_debug ("CdDeviceDb: add device %s [%s=%s]", device_id, property, value);
-	statement = sqlite3_mprintf ("INSERT INTO properties (device_id, "
+	statement = sqlite3_mprintf ("INSERT OR REPLACE INTO properties (device_id, "
 				     "property, value) "
 				     "VALUES ('%q', '%q', '%q')",
 				     device_id, property, value);
@@ -200,8 +202,8 @@ cd_device_db_set_property (CdDeviceDb *ddb,
 	rc = sqlite3_exec (ddb->priv->db, statement, NULL, NULL, &error_msg);
 	if (rc != SQLITE_OK) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "SQL error: %s",
 			     error_msg);
 		sqlite3_free (error_msg);
@@ -238,8 +240,8 @@ cd_device_db_remove (CdDeviceDb *ddb,
 	rc = sqlite3_exec (ddb->priv->db, statement1, NULL, NULL, &error_msg);
 	if (rc != SQLITE_OK) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "SQL error: %s",
 			     error_msg);
 		sqlite3_free (error_msg);
@@ -252,8 +254,8 @@ cd_device_db_remove (CdDeviceDb *ddb,
 	rc = sqlite3_exec (ddb->priv->db, statement2, NULL, NULL, &error_msg);
 	if (rc != SQLITE_OK) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "SQL error: %s",
 			     error_msg);
 		sqlite3_free (error_msg);
@@ -316,8 +318,8 @@ cd_device_db_get_property (CdDeviceDb *ddb,
 			   &error_msg);
 	if (rc != SQLITE_OK) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "SQL error: %s",
 			     error_msg);
 		sqlite3_free (error_msg);
@@ -327,8 +329,8 @@ cd_device_db_get_property (CdDeviceDb *ddb,
 	/* never set */
 	if (array_tmp->len == 0) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "no such property %s for %s",
 			     property, device_id);
 		goto out;
@@ -369,8 +371,8 @@ cd_device_db_get_devices (CdDeviceDb *ddb,
 			   &error_msg);
 	if (rc != SQLITE_OK) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "SQL error: %s",
 			     error_msg);
 		sqlite3_free (error_msg);
@@ -415,8 +417,8 @@ cd_device_db_get_properties (CdDeviceDb *ddb,
 			   &error_msg);
 	if (rc != SQLITE_OK) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_CLIENT_ERROR,
+			     CD_CLIENT_ERROR_INTERNAL,
 			     "SQL error: %s",
 			     error_msg);
 		sqlite3_free (error_msg);
