@@ -72,6 +72,7 @@ struct _CdDevicePrivate
 	guint64				 modified;
 	gboolean			 require_modified_signal;
 	gboolean			 is_virtual;
+	gboolean			 enabled;
 	GHashTable			*metadata;
 	guint				 owner;
 	gchar				*seat;
@@ -98,6 +99,25 @@ typedef struct {
 static guint signals[SIGNAL_LAST] = { 0 };
 G_DEFINE_TYPE (CdDevice, cd_device, G_TYPE_OBJECT)
 
+/**
+ * cd_device_error_quark:
+ **/
+GQuark
+cd_device_error_quark (void)
+{
+	guint i;
+	static GQuark quark = 0;
+	if (!quark) {
+		quark = g_quark_from_static_string ("CdDevice");
+		for (i = 0; i < CD_DEVICE_ERROR_LAST; i++) {
+			g_dbus_error_register_error (quark,
+						     i,
+						     cd_device_error_to_string (i));
+		}
+	}
+	return quark;
+}
+
 #if !GLIB_CHECK_VERSION (2, 25, 0)
 static guint64
 g_get_real_time (void)
@@ -294,6 +314,8 @@ cd_device_set_object_path (CdDevice *device)
 void
 cd_device_set_id (CdDevice *device, const gchar *id)
 {
+	gchar *enabled_str;
+
 	g_return_if_fail (CD_IS_DEVICE (device));
 
 	g_free (device->priv->id);
@@ -301,6 +323,19 @@ cd_device_set_id (CdDevice *device, const gchar *id)
 
 	/* now calculate this again */
 	cd_device_set_object_path (device);
+
+	/* find initial enabled state */
+	enabled_str = cd_device_db_get_property (device->priv->device_db,
+						 device->priv->id,
+						 "Enabled",
+						 NULL);
+	if (g_strcmp0 (enabled_str, "False") == 0) {
+		g_debug ("%s disabled by db at load", id);
+		device->priv->enabled = FALSE;
+	} else {
+		device->priv->enabled = TRUE;
+	}
+	g_free (enabled_str);
 }
 
 /**
@@ -406,7 +441,7 @@ cd_device_match_qualifier (const gchar *qual1, const gchar *qual2)
 	split2 = g_strsplit (qual2, ".", 3);
 
 	/* ensure all substrings match */
-	for (i=0; i<3; i++) {
+	for (i = 0; i < 3; i++) {
 
 		/* wildcard in query */
 		if (g_strcmp0 (split1[i], "*") == 0)
@@ -447,7 +482,7 @@ cd_device_find_by_qualifier (const gchar *regex,
 	guint i;
 
 	/* find using a wildcard */
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		item = g_ptr_array_index (array, i);
 		if (item->relation != relation)
 			continue;
@@ -494,7 +529,7 @@ cd_device_find_profile_by_object_path (GPtrArray *array, const gchar *object_pat
 	guint i;
 
 	/* find using an object path */
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		item = g_ptr_array_index (array, i);
 		ret = (g_strcmp0 (object_path,
 				  cd_profile_get_object_path (item->profile)) == 0);
@@ -522,14 +557,14 @@ cd_device_get_profiles_as_variant (CdDevice *device)
 
 	/* copy the object paths, hard then soft */
 	profiles = g_new0 (GVariant *, device->priv->profiles->len + 1);
-	for (i=0; i<device->priv->profiles->len; i++) {
+	for (i = 0; i < device->priv->profiles->len; i++) {
 		item = g_ptr_array_index (device->priv->profiles, i);
 		if (item->relation == CD_DEVICE_RELATION_SOFT)
 			continue;
 		tmp = cd_profile_get_object_path (item->profile);
 		profiles[idx++] = g_variant_new_object_path (tmp);
 	}
-	for (i=0; i<device->priv->profiles->len; i++) {
+	for (i = 0; i < device->priv->profiles->len; i++) {
 		item = g_ptr_array_index (device->priv->profiles, i);
 		if (item->relation == CD_DEVICE_RELATION_HARD)
 			continue;
@@ -558,8 +593,17 @@ cd_device_remove_profile (CdDevice *device,
 	gboolean ret = FALSE;
 	guint i;
 
+	/* device is disabled */
+	if (priv->enabled == FALSE) {
+		g_set_error_literal (error,
+				     CD_DEVICE_ERROR,
+				     CD_DEVICE_ERROR_NOT_ENABLED,
+				     "device is disabled");
+		goto out;
+	}
+
 	/* check the profile exists on this device */
-	for (i=0; i<priv->profiles->len; i++) {
+	for (i = 0; i < priv->profiles->len; i++) {
 		item = g_ptr_array_index (priv->profiles, i);
 		if (g_strcmp0 (profile_object_path,
 			       cd_profile_get_object_path (item->profile)) == 0) {
@@ -569,8 +613,8 @@ cd_device_remove_profile (CdDevice *device,
 	}
 	if (!ret) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_DEVICE_ERROR,
+			     CD_DEVICE_ERROR_PROFILE_DOES_NOT_EXIST,
 			     "profile object path '%s' does not exist on '%s'",
 			     profile_object_path,
 			     priv->object_path);
@@ -608,7 +652,7 @@ cd_device_find_profile_relation (CdDevice *device,
 	guint i;
 
 	/* search profiles */
-	for (i=0; i<priv->profiles->len; i++) {
+	for (i = 0; i < priv->profiles->len; i++) {
 		item = g_ptr_array_index (priv->profiles, i);
 		if (g_strcmp0 (profile_object_path,
 			       cd_profile_get_object_path (item->profile)) == 0) {
@@ -663,32 +707,42 @@ cd_device_add_profile (CdDevice *device,
 {
 	CdDevicePrivate *priv = device->priv;
 	CdDeviceProfileItem *item;
-	CdProfile *profile;
+	CdProfile *profile = NULL;
 	gboolean ret = TRUE;
 	guint i;
 
+	/* device is disabled */
+	if (priv->enabled == FALSE) {
+		ret = FALSE;
+		g_set_error_literal (error,
+				     CD_DEVICE_ERROR,
+				     CD_DEVICE_ERROR_NOT_ENABLED,
+				     "device is disabled");
+		goto out;
+	}
+
 	/* is it available */
 	profile = cd_profile_array_get_by_object_path (priv->profile_array,
 						       profile_object_path);
 	if (profile == NULL) {
 		ret = FALSE;
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_DEVICE_ERROR,
+			     CD_DEVICE_ERROR_PROFILE_DOES_NOT_EXIST,
 			     "profile object path '%s' does not exist",
 			     profile_object_path);
 		goto out;
 	}
 
 	/* check it does not already exist */
-	for (i=0; i<priv->profiles->len; i++) {
+	for (i = 0; i < priv->profiles->len; i++) {
 		item = g_ptr_array_index (priv->profiles, i);
 		if (g_strcmp0 (cd_profile_get_object_path (profile),
 			       cd_profile_get_object_path (item->profile)) == 0) {
 			ret = FALSE;
 			g_set_error (error,
-				     CD_MAIN_ERROR,
-				     CD_MAIN_ERROR_FAILED,
+				     CD_DEVICE_ERROR,
+				     CD_DEVICE_ERROR_PROFILE_ALREADY_ADDED,
 				     "profile object path '%s' has already been added",
 				     profile_object_path);
 			goto out;
@@ -961,11 +1015,22 @@ cd_device_make_default (CdDevice *device,
 	gboolean ret = FALSE;
 	CdDevicePrivate *priv = device->priv;
 
+	/* device is disabled */
+	if (priv->enabled == FALSE) {
+		g_set_error_literal (error,
+				     CD_DEVICE_ERROR,
+				     CD_DEVICE_ERROR_NOT_ENABLED,
+				     "device is disabled");
+		goto out;
+	}
+
 	/* find profile */
 	profile = cd_device_find_profile_by_object_path (priv->profiles,
 							 profile_object_path);
 	if (profile == NULL) {
-		g_set_error (error, 1, 0,
+		g_set_error (error,
+			     CD_DEVICE_ERROR,
+			     CD_DEVICE_ERROR_PROFILE_DOES_NOT_EXIST,
 			     "profile object path '%s' does not exist for this device",
 			     profile_object_path);
 		goto out;
@@ -1000,11 +1065,61 @@ out:
 	return ret;
 }
 
+/**
+ * cd_device_set_enabled:
+ **/
+static gboolean
+cd_device_set_enabled (CdDevice *device,
+		       gboolean enabled,
+		       GError **error)
+{
+	CdDevicePrivate *priv = device->priv;
+	gboolean ret;
+	GError *error_local = NULL;
+
+	/* device is already the correct state */
+	if (priv->enabled == enabled) {
+		ret = TRUE;
+		goto out;
+	}
+
+	/* update database */
+	ret = cd_device_db_set_property (device->priv->device_db,
+					 device->priv->id,
+					 "Enabled",
+					 enabled ? "True" : "False",
+					 &error_local);
+	if (!ret) {
+		g_set_error (error,
+			     CD_DEVICE_ERROR,
+			     CD_DEVICE_ERROR_INTERNAL,
+			     "%s", error_local->message);
+		g_error_free (error_local);
+		goto out;
+	}
+
+	/* change property */
+	priv->enabled = enabled;
+
+	/* reset modification time */
+	cd_device_reset_modified (device);
+
+	/* emit */
+	cd_device_dbus_emit_property_changed (device,
+					      "Enabled",
+					      g_variant_new_boolean (enabled));
+
+	/* emit global signal */
+	cd_device_dbus_emit_device_changed (device);
+out:
+	return ret;
+}
+
 /**
  * cd_device_dbus_method_call:
  **/
 static void
-cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
+cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 			    const gchar *object_path, const gchar *interface_name,
 			    const gchar *method_name, GVariant *parameters,
 			    GDBusMethodInvocation *invocation, gpointer user_data)
@@ -1013,6 +1128,7 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 	CdDevicePrivate *priv = device->priv;
 	CdProfile *profile = NULL;
 	const gchar *id;
+	gboolean enabled;
 	gboolean ret;
 	const gchar *profile_object_path = NULL;
 	const gchar *property_name = NULL;
@@ -1020,7 +1136,6 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 	gchar **regexes = NULL;
 	GError *error = NULL;
 	guint i = 0;
-	GVariantIter *iter = NULL;
 	GVariant *tuple = NULL;
 	GVariant *value = NULL;
 	gchar *tmp;
@@ -1030,10 +1145,18 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 	if (g_strcmp0 (method_name, "AddProfile") == 0) {
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.modify-device");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.modify-device",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
+							       "%s", error->message);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* check the profile_object_path exists */
 		g_variant_get (parameters, "(&s&o)",
@@ -1051,8 +1174,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		/* nothing valid */
 		if (relation == CD_DEVICE_RELATION_UNKNOWN) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_INTERNAL,
 							       "relation '%s' unknown, expected 'hard' or 'soft'",
 							       property_value);
 			goto out;
@@ -1065,10 +1188,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 					     g_get_real_time (),
 					     &error);
 		if (!ret) {
-			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
-							       "%s", error->message);
+			g_dbus_method_invocation_return_gerror (invocation,
+								error);
 			g_error_free (error);
 			goto out;
 		}
@@ -1099,10 +1220,18 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 	if (g_strcmp0 (method_name, "RemoveProfile") == 0) {
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.modify-device");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.modify-device",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
+							       "%s", error->message);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* try to remove */
 		g_variant_get (parameters, "(&o)",
@@ -1113,10 +1242,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 						profile_object_path,
 						&error);
 		if (!ret) {
-			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
-							       "%s", error->message);
+			g_dbus_method_invocation_return_gerror (invocation,
+								error);
 			g_error_free (error);
 			goto out;
 		}
@@ -1154,8 +1281,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 							    property_value);
 		if (relation == CD_DEVICE_RELATION_UNKNOWN) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_PROFILE_DOES_NOT_EXIST,
 							       "no profile '%s' found",
 							       property_value);
 			goto out;
@@ -1184,8 +1311,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		if (!ret) {
 			g_debug ("CdDevice: returning no results for profiling");
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_PROFILING,
 							       "profiling, so ignoring '%s'",
 							       property_name);
 			goto out;
@@ -1208,8 +1335,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		}
 		if (profile == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_NOTHING_MATCHED,
 							       "nothing matched expression '%s'",
 							       property_name);
 			goto out;
@@ -1225,10 +1352,18 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 	if (g_strcmp0 (method_name, "MakeProfileDefault") == 0) {
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.modify-device");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.modify-device",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
+							       "%s", error->message);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* check the profile_object_path exists */
 		g_variant_get (parameters, "(&o)",
@@ -1241,11 +1376,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 					      profile_object_path,
 					      &error);
 		if (!ret) {
-			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
-							       "failed to make profile default: %s",
-							       error->message);
+			g_dbus_method_invocation_return_gerror (invocation,
+								error);
 			g_error_free (error);
 			goto out;
 		}
@@ -1271,14 +1403,55 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		goto out;
 	}
 
+	/* return '' */
+	if (g_strcmp0 (method_name, "SetEnabled") == 0) {
+
+		/* require auth */
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.modify-device",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
+							       "%s", error->message);
+			g_error_free (error);
+			goto out;
+		}
+
+		/* set, and parse */
+		g_variant_get (parameters, "(b)",
+			       &enabled);
+		g_debug ("CdDevice %s:SetEnabled(%s)",
+			 sender, enabled ? "True" : "False");
+		ret = cd_device_set_enabled (device, enabled, &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_gerror (invocation,
+							        error);
+			g_error_free (error);
+			goto out;
+		}
+		g_dbus_method_invocation_return_value (invocation, NULL);
+		goto out;
+	}
+
 	/* return '' */
 	if (g_strcmp0 (method_name, "SetProperty") == 0) {
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.modify-device");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.modify-device",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
+							       "%s", error->message);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* set, and parse */
 		g_variant_get (parameters, "(&s&s)",
@@ -1292,10 +1465,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 						       (priv->object_scope == CD_OBJECT_SCOPE_DISK),
 						       &error);
 		if (!ret) {
-			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
-							       "%s", error->message);
+			g_dbus_method_invocation_return_gerror (invocation,
+								error);
 			g_error_free (error);
 			goto out;
 		}
@@ -1307,10 +1478,18 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 	if (g_strcmp0 (method_name, "ProfilingInhibit") == 0) {
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.device-inhibit");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.device-inhibit",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
+							       "%s", error->message);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* inhbit all profiles */
 		g_debug ("CdDevice %s:ProfilingInhibit()",
@@ -1320,8 +1499,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 				      &error);
 		if (!ret) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_FAILED_TO_INHIBIT,
 							       "%s", error->message);
 			g_error_free (error);
 			goto out;
@@ -1341,8 +1520,8 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 					 &error);
 		if (!ret) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_DEVICE_ERROR,
+							       CD_DEVICE_ERROR_FAILED_TO_UNINHIBIT,
 							       "%s", error->message);
 			g_error_free (error);
 			goto out;
@@ -1354,8 +1533,7 @@ cd_device_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 	/* we suck */
 	g_critical ("failed to process device method %s", method_name);
 out:
-	if (iter != NULL)
-		g_variant_iter_free (iter);
+	return;
 }
 
 /**
@@ -1414,6 +1592,10 @@ cd_device_dbus_get_property (GDBusConnection *connection_, const gchar *sender,
 		retval = cd_device_get_nullable_for_string (priv->serial);
 		goto out;
 	}
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_ENABLED) == 0) {
+		retval = g_variant_new_boolean (priv->enabled);
+		goto out;
+	}
 	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_COLORSPACE) == 0) {
 		retval = cd_device_get_nullable_for_string (priv->colorspace);
 		goto out;
@@ -1441,6 +1623,9 @@ cd_device_dbus_get_property (GDBusConnection *connection_, const gchar *sender,
 		if (!ret) {
 			g_debug ("CdDevice: returning no profiles for profiling");
 			retval = g_variant_new ("ao", NULL);
+		} else if (!priv->enabled) {
+			g_debug ("CdDevice: returning no profiles as device disabled");
+			retval = g_variant_new ("ao", NULL);
 		} else {
 			retval = cd_device_get_profiles_as_variant (device);
 		}
@@ -1503,8 +1688,8 @@ cd_device_register_object (CdDevice *device,
 		&error_local); /* GError** */
 	if (device->priv->registration_id == 0) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_DEVICE_ERROR,
+			     CD_DEVICE_ERROR_INTERNAL,
 			     "failed to register object: %s",
 			     error_local->message);
 		g_error_free (error_local);
