@@ -26,6 +26,7 @@
 #include <gio/gunixfdlist.h>
 #include <glib/gi18n.h>
 #include <locale.h>
+#include <syslog.h>
 
 #ifdef HAVE_LIBSYSTEMD_LOGIN
 #include <systemd/sd-login.h>
@@ -82,12 +83,15 @@ cd_main_profile_removed (CdMainPrivate *priv, CdProfile *profile)
 
 	/* try to remove this profile from all devices */
 	devices = cd_device_array_get_array (priv->devices_array);
-	for (i=0; i<devices->len; i++) {
+	for (i = 0; i < devices->len; i++) {
 		device_tmp = g_ptr_array_index (devices, i);
 		ret = cd_device_remove_profile (device_tmp,
 						object_path_tmp,
 						NULL);
 		if (ret) {
+			syslog (LOG_INFO, "Automatic remove of %s from %s",
+				cd_profile_get_id (profile),
+				cd_device_get_id (device_tmp));
 			g_debug ("CdMain: automatically removing %s from %s as removed",
 				 object_path_tmp,
 				 cd_device_get_object_path (device_tmp));
@@ -96,6 +100,8 @@ cd_main_profile_removed (CdMainPrivate *priv, CdProfile *profile)
 
 	/* emit signal */
 	g_debug ("CdMain: Emitting ProfileRemoved(%s)", object_path_tmp);
+	syslog (LOG_INFO, "Profile removed: %s",
+		cd_profile_get_id (profile));
 	g_dbus_connection_emit_signal (priv->connection,
 				       NULL,
 				       COLORD_DBUS_PATH,
@@ -152,6 +158,8 @@ cd_main_device_removed (CdMainPrivate *priv, CdDevice *device)
 
 	/* emit signal */
 	g_debug ("CdMain: Emitting DeviceRemoved(%s)", object_path_tmp);
+	syslog (LOG_INFO, "device removed: %s",
+		cd_device_get_id (device));
 	g_dbus_connection_emit_signal (priv->connection,
 				       NULL,
 				       COLORD_DBUS_PATH,
@@ -240,8 +248,8 @@ cd_main_create_profile (CdMainPrivate *priv,
 	case CD_OBJECT_SCOPE_DISK:
 		g_debug ("CdMain: persistent profile");
 		g_set_error_literal (error,
-				     CD_MAIN_ERROR,
-				     CD_MAIN_ERROR_FAILED,
+				     CD_CLIENT_ERROR,
+				     CD_CLIENT_ERROR_NOT_SUPPORTED,
 				     "persistent profiles are no yet supported");
 		goto out;
 	default:
@@ -280,6 +288,9 @@ cd_main_device_auto_add_profile_md (CdDevice *device, CdProfile *profile)
 	g_debug ("CdMain: Automatically MD add %s to %s",
 		 cd_profile_get_id (profile),
 		 cd_device_get_object_path (device));
+	syslog (LOG_INFO, "Automatic metadata add %s to %s",
+		cd_profile_get_id (profile),
+		cd_device_get_id (device));
 	ret = cd_device_add_profile (device,
 				     CD_DEVICE_RELATION_SOFT,
 				     cd_profile_get_object_path (profile),
@@ -310,6 +321,9 @@ cd_main_device_auto_add_profile_db (CdMainPrivate *priv,
 	g_debug ("CdMain: Automatically DB add %s to %s",
 		 cd_profile_get_id (profile),
 		 cd_device_get_object_path (device));
+	syslog (LOG_INFO, "Automatic database add %s to %s",
+		cd_profile_get_id (profile),
+		cd_device_get_id (device));
 	timestamp = cd_mapping_db_get_timestamp (priv->mapping_db,
 						 cd_device_get_id (device),
 						 cd_profile_get_id (profile),
@@ -385,7 +399,7 @@ cd_main_device_auto_add_profiles (CdMainPrivate *priv,
 	}
 
 	/* try to add them */
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		object_id_tmp = g_ptr_array_index (array, i);
 		profile_tmp = cd_profile_array_get_by_id_owner (priv->profiles_array,
 								object_id_tmp,
@@ -426,6 +440,8 @@ cd_main_device_register_on_bus (CdMainPrivate *priv,
 	/* emit signal */
 	g_debug ("CdMain: Emitting DeviceAdded(%s)",
 		 cd_device_get_object_path (device));
+	syslog (LOG_INFO, "Device added: %s",
+		cd_device_get_id (device));
 	g_dbus_connection_emit_signal (priv->connection,
 				       NULL,
 				       COLORD_DBUS_PATH,
@@ -566,7 +582,7 @@ cd_main_device_array_to_variant (GPtrArray *array)
 
 	/* copy the object paths */
 	variant_array = g_new0 (GVariant *, array->len + 1);
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		device = g_ptr_array_index (array, i);
 		variant_array[length] = g_variant_new_object_path (
 			cd_device_get_object_path (device));
@@ -595,7 +611,7 @@ cd_main_profile_array_to_variant (GPtrArray *array)
 
 	/* copy the object paths */
 	variant_array = g_new0 (GVariant *, array->len + 1);
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		profile = g_ptr_array_index (array, i);
 		variant_array[length] = g_variant_new_object_path (
 			cd_profile_get_object_path (profile));
@@ -624,7 +640,7 @@ cd_main_sensor_array_to_variant (GPtrArray *array)
 
 	/* copy the object paths */
 	variant_array = g_new0 (GVariant *, array->len + 1);
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		sensor = g_ptr_array_index (array, i);
 		variant_array[length] = g_variant_new_object_path (
 			cd_sensor_get_object_path (sensor));
@@ -671,7 +687,7 @@ cd_main_profile_auto_add_to_device (CdMainPrivate *priv,
 	}
 
 	/* try to add them */
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		device_id_tmp = g_ptr_array_index (array, i);
 		device_tmp = cd_device_array_get_by_id_owner (priv->devices_array,
 							      device_id_tmp,
@@ -712,6 +728,8 @@ cd_main_profile_register_on_bus (CdMainPrivate *priv,
 	/* emit signal */
 	g_debug ("CdMain: Emitting ProfileAdded(%s)",
 		 cd_profile_get_object_path (profile));
+	syslog (LOG_INFO, "Profile added: %s",
+		cd_profile_get_id (profile));
 	g_dbus_connection_emit_signal (priv->connection,
 				       NULL,
 				       COLORD_DBUS_PATH,
@@ -756,11 +774,11 @@ cd_main_get_standard_space_metadata (CdMainPrivate *priv,
 
 	/* get all the profiles with this metadata */
 	array = cd_profile_array_get_by_metadata (priv->profiles_array,
-						  "STANDARD_space",
+						  CD_PROFILE_METADATA_STANDARD_SPACE,
 						  standard_space);
 
 	/* just use the first system-wide profile */
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		profile_tmp = g_ptr_array_index (array, i);
 		ret = cd_profile_get_is_system_wide (profile_tmp);
 		if (ret) {
@@ -777,7 +795,7 @@ out:
  * cd_main_daemon_method_call:
  **/
 static void
-cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
+cd_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 			    const gchar *object_path, const gchar *interface_name,
 			    const gchar *method_name, GVariant *parameters,
 			    GDBusMethodInvocation *invocation, gpointer user_data)
@@ -786,6 +804,7 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	CdMainPrivate *priv = (CdMainPrivate *) user_data;
 	CdObjectScope scope;
 	CdProfile *profile = NULL;
+	CdSensor *sensor_tmp;
 	const gchar *prop_key;
 	const gchar *prop_value;
 	gboolean register_on_bus = TRUE;
@@ -798,6 +817,7 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	GVariant *tuple = NULL;
 	GVariant *value = NULL;
 	gint fd = -1;
+	guint i;
 	guint pid;
 	guint uid;
 	gint32 fd_handle = 0;
@@ -807,11 +827,11 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	GUnixFDList *fd_list;
 
 	/* get the owner of the message */
-	uid = cd_main_get_sender_uid (invocation, &error);
+	uid = cd_main_get_sender_uid (connection, sender, &error);
 	if (uid == G_MAXUINT) {
 		g_dbus_method_invocation_return_error (invocation,
-						       CD_MAIN_ERROR,
-						       CD_MAIN_ERROR_FAILED,
+						       CD_CLIENT_ERROR,
+						       CD_CLIENT_ERROR_INTERNAL,
 						       "failed to get owner: %s",
 						       error->message);
 		g_error_free (error);
@@ -888,8 +908,8 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 							  uid);
 		if (device == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_NOT_FOUND,
 							       "device id '%s' does not exists",
 							       device_id);
 			goto out;
@@ -914,8 +934,8 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 							  metadata_value);
 		if (device == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_NOT_FOUND,
 							       "property match '%s'='%s' does not exist",
 							       metadata_key,
 							       metadata_value);
@@ -928,6 +948,30 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		goto out;
 	}
 
+	/* return 'o' */
+	if (g_strcmp0 (method_name, "FindSensorById") == 0) {
+
+		g_variant_get (parameters, "(&s)", &device_id);
+		g_debug ("CdMain: %s:FindSensorById(%s)",
+			 sender, device_id);
+
+		/* find sensor */
+		for (i = 0; i < priv->sensors->len; i++) {
+			sensor_tmp = g_ptr_array_index (priv->sensors, i);
+			if (g_strcmp0 (cd_sensor_get_id (sensor_tmp), device_id) == 0) {
+				value = g_variant_new ("(o)", cd_sensor_get_object_path (sensor_tmp));
+				g_dbus_method_invocation_return_value (invocation, value);
+				goto out;
+			}
+		}
+		g_dbus_method_invocation_return_error (invocation,
+						       CD_CLIENT_ERROR,
+						       CD_CLIENT_ERROR_NOT_FOUND,
+						       "sensor id '%s' does not exist",
+						       device_id);
+		goto out;
+	}
+
 	/* return 'o' */
 	if (g_strcmp0 (method_name, "FindProfileByProperty") == 0) {
 
@@ -941,8 +985,8 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 							    metadata_value);
 		if (profile == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_NOT_FOUND,
 							       "property match '%s'='%s' does not exist",
 							       metadata_key,
 							       metadata_value);
@@ -966,8 +1010,8 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 							    uid);
 		if (profile == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_NOT_FOUND,
 							       "profile id '%s' does not exist",
 							       device_id);
 			goto out;
@@ -992,8 +1036,8 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 			profile = cd_main_get_standard_space_metadata (priv, device_id);
 		if (profile == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_NOT_FOUND,
 							       "profile space '%s' does not exist",
 							       device_id);
 			goto out;
@@ -1015,8 +1059,8 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 							    device_id);
 		if (profile == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_NOT_FOUND,
 							       "profile filename '%s' does not exist",
 							       device_id);
 			goto out;
@@ -1043,10 +1087,16 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	if (g_strcmp0 (method_name, "CreateDevice") == 0) {
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.create-device");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.create-device",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_gerror (invocation,
+								error);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* does already exist */
 		g_variant_get (parameters, "(&s&sa{ss})",
@@ -1055,8 +1105,25 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 			       &iter);
 		g_debug ("CdMain: %s:CreateDevice(%s)", sender, device_id);
 
+		/* check ID is valid */
+		if (g_strcmp0 (device_id, "") == 0) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_INPUT_INVALID,
+							       "device id cannot be blank");
+			goto out;
+		}
+
 		/* check it does not already exist */
 		scope = cd_object_scope_from_string (scope_tmp);
+		if (scope == CD_OBJECT_SCOPE_UNKNOWN) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_INPUT_INVALID,
+							       "scope non-valid: %s",
+							       scope_tmp);
+			goto out;
+		}
 		device = cd_device_array_get_by_id_owner (priv->devices_array,
 							  device_id,
 							  uid);
@@ -1070,8 +1137,8 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 				register_on_bus = FALSE;
 			} else {
 				g_dbus_method_invocation_return_error (invocation,
-								       CD_MAIN_ERROR,
-								       CD_MAIN_ERROR_ALREADY_EXISTS,
+								       CD_CLIENT_ERROR,
+								       CD_CLIENT_ERROR_ALREADY_EXISTS,
 								       "device id '%s' already exists",
 								       device_id);
 				goto out;
@@ -1079,11 +1146,11 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		}
 
 		/* get the process that sent the message */
-		pid = cd_main_get_sender_pid (invocation, &error);
+		pid = cd_main_get_sender_pid (connection, sender, &error);
 		if (pid == G_MAXUINT) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_INTERNAL,
 							       "failed to get process ID: %s",
 							       error->message);
 			g_error_free (error);
@@ -1148,10 +1215,16 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	if (g_strcmp0 (method_name, "DeleteDevice") == 0) {
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.delete-device");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.delete-device",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_gerror (invocation,
+								error);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* does already exist */
 		g_variant_get (parameters, "(&o)", &device_id);
@@ -1166,8 +1239,8 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 								     device_id);
 			if (device == NULL) {
 				g_dbus_method_invocation_return_error (invocation,
-								       CD_MAIN_ERROR,
-								       CD_MAIN_ERROR_FAILED,
+								       CD_CLIENT_ERROR,
+								       CD_CLIENT_ERROR_NOT_FOUND,
 								       "device path '%s' not found",
 								       device_id);
 				goto out;
@@ -1185,10 +1258,16 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	if (g_strcmp0 (method_name, "DeleteProfile") == 0) {
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.create-profile");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.create-profile",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_gerror (invocation,
+								error);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* does already exist */
 		g_variant_get (parameters, "(&o)", &device_id);
@@ -1203,8 +1282,8 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 								       device_id);
 			if (profile == NULL) {
 				g_dbus_method_invocation_return_error (invocation,
-								       CD_MAIN_ERROR,
-								       CD_MAIN_ERROR_FAILED,
+								       CD_CLIENT_ERROR,
+								       CD_CLIENT_ERROR_NOT_FOUND,
 								       "profile path '%s' not found",
 								       device_id);
 				goto out;
@@ -1223,10 +1302,16 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	    g_strcmp0 (method_name, "CreateProfileWithFd") == 0) {
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.create-profile");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.create-profile",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_gerror (invocation,
+								error);
+			g_error_free (error);
 			goto out;
+		}
 
 		if (g_strcmp0 (g_variant_get_type_string (parameters),
 			       "(ssha{ss})") == 0) {
@@ -1246,14 +1331,23 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 			g_debug ("CdMain: %s:CreateProfile(%s)", sender, device_id);
 		}
 
+		/* check ID is valid */
+		if (g_strcmp0 (device_id, "") == 0) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_INPUT_INVALID,
+							       "profile id cannot be blank");
+			goto out;
+		}
+
 		/* check it does not already exist */
 		profile = cd_profile_array_get_by_id_owner (priv->profiles_array,
 							    device_id,
 							    uid);
 		if (profile != NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_ALREADY_EXISTS,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_ALREADY_EXISTS,
 							       "profile id '%s' already exists",
 							       device_id);
 			goto out;
@@ -1261,6 +1355,14 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 
 		/* create profile */
 		scope = cd_object_scope_from_string (scope_tmp);
+		if (scope == CD_OBJECT_SCOPE_UNKNOWN) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_CLIENT_ERROR,
+							       CD_CLIENT_ERROR_INPUT_INVALID,
+							       "scope non-valid: %s",
+							       scope_tmp);
+			goto out;
+		}
 		profile = cd_main_create_profile (priv,
 						  sender,
 						  device_id,
@@ -1487,7 +1589,7 @@ cd_main_add_disk_device (CdMainPrivate *priv, const gchar *device_id)
 		g_error_free (error);
 		goto out;
 	}
-	for (i=0; i<array_properties->len; i++) {
+	for (i = 0; i < array_properties->len; i++) {
 		property = g_ptr_array_index (array_properties, i);
 		value = cd_device_db_get_property (priv->device_db,
 						   device_id,
@@ -1549,6 +1651,8 @@ cd_main_sensor_register_on_bus (CdMainPrivate *priv,
 	/* emit signal */
 	g_debug ("CdMain: Emitting SensorAdded(%s)",
 		 cd_sensor_get_object_path (sensor));
+	syslog (LOG_INFO, "Sensor added: %s",
+		cd_sensor_get_id (sensor));
 	g_dbus_connection_emit_signal (priv->connection,
 				       NULL,
 				       COLORD_DBUS_PATH,
@@ -1836,6 +1940,8 @@ cd_main_client_sensor_removed_cb (CdSensorClient *sensor_client_,
 	/* emit signal */
 	g_debug ("CdMain: Emitting SensorRemoved(%s)",
 		 cd_sensor_get_object_path (sensor));
+	syslog (LOG_INFO, "Sensor removed: %s",
+		cd_sensor_get_id (sensor));
 	g_dbus_connection_emit_signal (priv->connection,
 				       NULL,
 				       COLORD_DBUS_PATH,
@@ -2020,6 +2126,7 @@ cd_main_load_plugins (CdMainPrivate *priv)
 						    filename_tmp,
 						    NULL);
 		cd_main_load_plugin (priv, filename_plugin);
+		syslog (LOG_INFO, "Loading plugin %s", filename_tmp);
 		g_free (filename_plugin);
 	} while (TRUE);
 out:
@@ -2054,6 +2161,7 @@ main (int argc, char *argv[])
 	};
 
 	setlocale (LC_ALL, "");
+	openlog ("colord", LOG_CONS, LOG_DAEMON);
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -2178,10 +2286,12 @@ main (int argc, char *argv[])
 	cd_main_plugin_phase (priv, CD_PLUGIN_PHASE_INIT);
 
 	/* wait */
+	syslog (LOG_INFO, "Daemon ready for requests");
 	g_main_loop_run (loop);
 
 	/* run the plugins */
 	cd_main_plugin_phase (priv, CD_PLUGIN_PHASE_DESTROY);
+	closelog ();
 
 	/* success */
 	retval = 0;
