@@ -52,6 +52,28 @@ static guint signals[SIGNAL_LAST] = { 0 };
 
 G_DEFINE_TYPE (CdSensorClient, cd_sensor_client, G_TYPE_OBJECT)
 
+/**
+ * cd_sensor_client_get_by_id:
+ **/
+CdSensor *
+cd_sensor_client_get_by_id (CdSensorClient *sensor_client,
+			    const gchar *sensor_id)
+{
+	CdSensorClientPrivate *priv = sensor_client->priv;
+	CdSensor *sensor = NULL;
+	CdSensor *sensor_tmp;
+	guint i;
+
+	for (i = 0; i < priv->array_sensors->len; i++) {
+		sensor_tmp = g_ptr_array_index (priv->array_sensors, i);
+		if (g_strcmp0 (cd_sensor_get_id (sensor_tmp), sensor_id) == 0) {
+			sensor = g_object_ref (sensor_tmp);
+			break;
+		}
+	}
+	return sensor;
+}
+
 /**
  * cd_sensor_client_add:
  **/
@@ -145,7 +167,7 @@ cd_sensor_client_remove (CdSensorClient *sensor_client,
 	device_path = g_udev_device_get_sysfs_path (device);
 	g_debug ("removing color management device: %s [%s]",
 		 device_path, device_file);
-	for (i=0; i<sensor_client->priv->array_sensors->len; i++) {
+	for (i = 0; i < sensor_client->priv->array_sensors->len; i++) {
 		sensor = g_ptr_array_index (sensor_client->priv->array_sensors, i);
 		if (g_strcmp0 (cd_sensor_get_device_path (sensor), device_path) == 0) {
 			g_debug ("emit: removed");
