@@ -91,6 +91,7 @@ struct _CdSensorPrivate
 	gchar				*vendor;
 	gchar				*device_path;
 	gboolean			 native;
+	gboolean			 embedded;
 	gboolean			 locked;
 	gchar				**caps;
 	gchar				*object_path;
@@ -119,6 +120,25 @@ enum {
 
 G_DEFINE_TYPE (CdSensor, cd_sensor, G_TYPE_OBJECT)
 
+/**
+ * cd_sensor_error_quark:
+ **/
+GQuark
+cd_sensor_error_quark (void)
+{
+	guint i;
+	static GQuark quark = 0;
+	if (!quark) {
+		quark = g_quark_from_static_string ("CdSensor");
+		for (i = 0; i < CD_SENSOR_ERROR_LAST; i++) {
+			g_dbus_error_register_error (quark,
+						     i,
+						     cd_sensor_error_to_string (i));
+		}
+	}
+	return quark;
+}
+
 /**
  * cd_sensor_get_object_path:
  **/
@@ -253,6 +273,15 @@ cd_sensor_set_kind (CdSensor *sensor, CdSensorKind kind)
 					      g_variant_new_uint32 (kind));
 }
 
+/**
+ * cd_sensor_get_kind:
+ **/
+CdSensorKind
+cd_sensor_get_kind (CdSensor *sensor)
+{
+	return sensor->priv->kind;
+}
+
 /**
  * cd_sensor_load:
  * @sensor: a valid #CdSensor instance
@@ -267,6 +296,7 @@ cd_sensor_load (CdSensor *sensor, GError **error)
 	gboolean ret = FALSE;
 	gchar *backend_name = NULL;
 	gchar *path = NULL;
+	gchar *path_fallback = NULL;
 	GModule *handle;
 
 	/* no module */
@@ -281,9 +311,17 @@ cd_sensor_load (CdSensor *sensor, GError **error)
 	g_debug ("Trying to load : %s", backend_name);
 	path = g_build_filename (LIBDIR, "colord-sensors", backend_name, NULL);
 	handle = g_module_open (path, G_MODULE_BIND_LOCAL);
+	if (handle == NULL) {
+		g_debug ("Trying to fall back to : libcolord_sensor_argyll");
+		path_fallback = g_build_filename (LIBDIR,
+						  "colord-sensors",
+						  "libcolord_sensor_argyll.so",
+						  NULL);
+		handle = g_module_open (path_fallback, G_MODULE_BIND_LOCAL);
+	}
 	if (handle == NULL) {
 		g_set_error (error, 1, 0,
-			     "opening module %s failed : %s",
+			     "opening module %s (and fallback) failed : %s",
 			     backend_name, g_module_error ());
 		goto out;
 	}
@@ -313,6 +351,7 @@ out:
 //		g_module_close (handle);
 	g_free (backend_name);
 	g_free (path);
+	g_free (path_fallback);
 	return ret;
 }
 
@@ -432,11 +471,8 @@ cd_sensor_get_sample_cb (GObject *source_object,
 	/* get the result */
 	sample = sensor->priv->desc->get_sample_finish (sensor, res, &error);
 	if (sample == NULL) {
-		g_dbus_method_invocation_return_error (invocation,
-						       CD_MAIN_ERROR,
-						       CD_MAIN_ERROR_FAILED,
-						       "failed to sample: %s",
-						       error->message);
+		g_dbus_method_invocation_return_gerror (invocation,
+							error);
 		g_error_free (error);
 		goto out;
 	}
@@ -472,11 +508,8 @@ cd_sensor_set_options_cb (GObject *source_object,
 	/* get the result */
 	ret = sensor->priv->desc->set_options_finish (sensor, res, &error);
 	if (!ret) {
-		g_dbus_method_invocation_return_error (invocation,
-						       CD_MAIN_ERROR,
-						       CD_MAIN_ERROR_FAILED,
-						       "failed to set options: %s",
-						       error->message);
+		g_dbus_method_invocation_return_gerror (invocation,
+							error);
 		g_error_free (error);
 		goto out;
 	}
@@ -502,8 +535,8 @@ cd_sensor_lock_cb (GObject *source_object,
 	ret = sensor->priv->desc->lock_finish (sensor, res, &error);
 	if (!ret) {
 		g_dbus_method_invocation_return_error (invocation,
-						       CD_MAIN_ERROR,
-						       CD_MAIN_ERROR_FAILED,
+						       CD_SENSOR_ERROR,
+						       CD_SENSOR_ERROR_NO_SUPPORT,
 						       "failed to lock: %s",
 						       error->message);
 		g_error_free (error);
@@ -529,15 +562,18 @@ cd_sensor_unlock_cb (GObject *source_object,
 	GError *error = NULL;
 
 	/* get the result */
-	ret = sensor->priv->desc->unlock_finish (sensor, res, &error);
-	if (!ret) {
-		g_dbus_method_invocation_return_error (invocation,
-						       CD_MAIN_ERROR,
-						       CD_MAIN_ERROR_FAILED,
-						       "failed to unlock: %s",
-						       error->message);
-		g_error_free (error);
-		goto out;
+	if (sensor->priv->desc != NULL &&
+	    sensor->priv->desc->unlock_finish != NULL) {
+		ret = sensor->priv->desc->unlock_finish (sensor, res, &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_NO_SUPPORT,
+							       "failed to unlock: %s",
+							       error->message);
+			g_error_free (error);
+			goto out;
+		}
 	}
 	cd_sensor_set_locked (sensor, FALSE);
 	g_dbus_method_invocation_return_value (invocation, NULL);
@@ -558,12 +594,15 @@ cd_sensor_unlock_quietly_cb (GObject *source_object,
 	GError *error = NULL;
 
 	/* get the result */
-	ret = sensor->priv->desc->unlock_finish (sensor, res, &error);
-	if (!ret) {
-		g_warning ("failed to unlock: %s",
-			   error->message);
-		g_error_free (error);
-		goto out;
+	if (sensor->priv->desc != NULL &&
+	    sensor->priv->desc->unlock_finish != NULL) {
+		ret = sensor->priv->desc->unlock_finish (sensor, res, &error);
+		if (!ret) {
+			g_warning ("failed to unlock: %s",
+				   error->message);
+			g_error_free (error);
+			goto out;
+		}
 	}
 	cd_sensor_set_locked (sensor, FALSE);
 out:
@@ -582,7 +621,8 @@ cd_sensor_name_vanished_cb (GDBusConnection *connection,
 
 	/* dummy */
 	g_debug ("locked sender has vanished without doing Unlock()!");
-	if (sensor->priv->desc->unlock_async == NULL) {
+	if (sensor->priv->desc == NULL ||
+	    sensor->priv->desc->unlock_async == NULL) {
 		cd_sensor_set_locked (sensor, FALSE);
 		goto out;
 	}
@@ -600,7 +640,7 @@ out:
  * cd_sensor_dbus_method_call:
  **/
 static void
-cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
+cd_sensor_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 			    const gchar *object_path, const gchar *interface_name,
 			    const gchar *method_name, GVariant *parameters,
 			    GDBusMethodInvocation *invocation, gpointer user_data)
@@ -611,20 +651,12 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 	const gchar *cap_tmp = NULL;
 	gboolean ret;
 	gchar *key;
+	GError *error = NULL;
 	GHashTable *options = NULL;
 	GVariantIter iter;
 	GVariant *result = NULL;
 	GVariant *value;
 
-	/* check native */
-	if (!priv->native) {
-		g_dbus_method_invocation_return_error (invocation,
-						       CD_MAIN_ERROR,
-						       CD_MAIN_ERROR_FAILED,
-						       "no native driver for sensor");
-		goto out;
-	}
-
 	/* return '' */
 	if (g_strcmp0 (method_name, "Lock") == 0) {
 
@@ -633,17 +665,25 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		/* check locked */
 		if (priv->locked) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_ALREADY_LOCKED,
 							       "sensor is already locked");
 			goto out;
 		}
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.sensor-lock");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.sensor-lock",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_FAILED_TO_AUTHENTICATE,
+							       "%s", error->message);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* watch this bus name */
 		priv->watcher_id = g_bus_watch_name (G_BUS_TYPE_SYSTEM,
@@ -655,7 +695,8 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 						     NULL);
 
 		/* no support */
-		if (sensor->priv->desc->lock_async == NULL) {
+		if (sensor->priv->desc == NULL ||
+		    sensor->priv->desc->lock_async == NULL) {
 			cd_sensor_set_locked (sensor, TRUE);
 			g_dbus_method_invocation_return_value (invocation, NULL);
 			goto out;
@@ -676,17 +717,25 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		/* check locked */
 		if (!priv->locked) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_NOT_LOCKED,
 							       "sensor is not yet locked");
 			goto out;
 		}
 
 		/* require auth */
-		ret = cd_main_sender_authenticated (invocation,
-						    "org.freedesktop.color-manager.sensor-lock");
-		if (!ret)
+		ret = cd_main_sender_authenticated (connection,
+						    sender,
+						    "org.freedesktop.color-manager.sensor-lock",
+						    &error);
+		if (!ret) {
+			g_dbus_method_invocation_return_error (invocation,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_FAILED_TO_AUTHENTICATE,
+							       "%s", error->message);
+			g_error_free (error);
 			goto out;
+		}
 
 		/* un-watch this bus name */
 		if (priv->watcher_id != 0) {
@@ -695,7 +744,8 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		}
 
 		/* no support */
-		if (sensor->priv->desc->unlock_async == NULL) {
+		if (sensor->priv->desc == NULL ||
+		    sensor->priv->desc->unlock_async == NULL) {
 			cd_sensor_set_locked (sensor, FALSE);
 			g_dbus_method_invocation_return_value (invocation, NULL);
 			goto out;
@@ -717,8 +767,8 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		/* check locked */
 		if (!priv->locked) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_NOT_LOCKED,
 							       "sensor is not yet locked");
 			goto out;
 		}
@@ -726,18 +776,19 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		/*  check idle */
 		if (priv->state != CD_SENSOR_STATE_IDLE) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_IN_USE,
 							       "sensor not idle: %s",
 							       cd_sensor_state_to_string (priv->state));
 			goto out;
 		}
 
 		/* no support */
-		if (sensor->priv->desc->get_sample_async == NULL) {
+		if (sensor->priv->desc == NULL ||
+		    sensor->priv->desc->get_sample_async == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_NO_SUPPORT,
 							       "no sensor->get_sample");
 			goto out;
 		}
@@ -747,8 +798,8 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		cap = cd_sensor_cap_from_string (cap_tmp);
 		if (cap == CD_SENSOR_CAP_UNKNOWN) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_INTERNAL,
 							       "cap '%s' unknown",
 							       cap_tmp);
 			goto out;
@@ -771,8 +822,8 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		/* check locked */
 		if (!priv->locked) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_NOT_LOCKED,
 							       "sensor is not yet locked");
 			goto out;
 		}
@@ -780,18 +831,19 @@ cd_sensor_dbus_method_call (GDBusConnection *connection_, const gchar *sender,
 		/*  check idle */
 		if (priv->state != CD_SENSOR_STATE_IDLE) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_IN_USE,
 							       "sensor not idle: %s",
 							       cd_sensor_state_to_string (priv->state));
 			goto out;
 		}
 
 		/* no support */
-		if (sensor->priv->desc->set_options_async == NULL) {
+		if (sensor->priv->desc == NULL ||
+		    sensor->priv->desc->set_options_async == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
+							       CD_SENSOR_ERROR,
+							       CD_SENSOR_ERROR_NO_SUPPORT,
 							       "no sensor options support");
 			goto out;
 		}
@@ -881,43 +933,51 @@ cd_sensor_dbus_get_property (GDBusConnection *connection_, const gchar *sender,
 
 	g_debug ("CdSensor %s:GetProperty '%s'",
 		 sender, property_name);
-	if (g_strcmp0 (property_name, "Kind") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_ID) == 0) {
+		retval = g_variant_new_string (priv->id);
+		goto out;
+	}
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_KIND) == 0) {
 		retval = g_variant_new_string (cd_sensor_kind_to_string (priv->kind));
 		goto out;
 	}
-	if (g_strcmp0 (property_name, "State") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_STATE) == 0) {
 		retval = g_variant_new_string (cd_sensor_state_to_string (priv->state));
 		goto out;
 	}
-	if (g_strcmp0 (property_name, "Mode") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_MODE) == 0) {
 		retval = g_variant_new_string (cd_sensor_cap_to_string (priv->mode));
 		goto out;
 	}
-	if (g_strcmp0 (property_name, "Serial") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_SERIAL) == 0) {
 		retval = cd_sensor_get_nullable_for_string (priv->serial);
 		goto out;
 	}
-	if (g_strcmp0 (property_name, "Model") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_MODEL) == 0) {
 		retval = cd_sensor_get_nullable_for_string (priv->model);
 		goto out;
 	}
-	if (g_strcmp0 (property_name, "Vendor") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_VENDOR) == 0) {
 		retval = cd_sensor_get_nullable_for_string (priv->vendor);
 		goto out;
 	}
-	if (g_strcmp0 (property_name, "Native") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_NATIVE) == 0) {
 		retval = g_variant_new_boolean (priv->native);
 		goto out;
 	}
-	if (g_strcmp0 (property_name, "Locked") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_LOCKED) == 0) {
 		retval = g_variant_new_boolean (priv->locked);
 		goto out;
 	}
-	if (g_strcmp0 (property_name, "Capabilities") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_EMBEDDED) == 0) {
+		retval = g_variant_new_boolean (priv->embedded);
+		goto out;
+	}
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_CAPABILITIES) == 0) {
 		retval = g_variant_new_strv ((const gchar * const*) priv->caps, -1);
 		goto out;
 	}
-	if (g_strcmp0 (property_name, "Options") == 0) {
+	if (g_strcmp0 (property_name, CD_SENSOR_PROPERTY_OPTIONS) == 0) {
 		retval = cd_sensor_get_options_as_variant (sensor);
 		goto out;
 	}
@@ -956,8 +1016,8 @@ cd_sensor_register_object (CdSensor *sensor,
 		&error_local); /* GError** */
 	if (sensor->priv->registration_id == 0) {
 		g_set_error (error,
-			     CD_MAIN_ERROR,
-			     CD_MAIN_ERROR_FAILED,
+			     CD_SENSOR_ERROR,
+			     CD_SENSOR_ERROR_INTERNAL,
 			     "failed to register object: %s",
 			     error_local->message);
 		g_error_free (error_local);
@@ -1069,6 +1129,12 @@ cd_sensor_set_from_device (CdSensor *sensor,
 		priv->caps[idx++] = g_strdup ("ambient");
 	priv->caps[idx] = NULL;
 
+	/* is the sensor embeded, e.g. on the W700? */
+	ret = g_udev_device_get_property_as_boolean (device,
+						     "COLORD_SENSOR_EMBEDDED");
+	if (ret)
+		priv->embedded = TRUE;
+
 	/* no caps */
 	if (idx == 0) {
 		ret = FALSE;
